@@ -81,7 +81,9 @@ function addClassName(element, name) {
 
 function elementSetStyle(element, style) {
     for (var name in style) {
-      element.style[name] = style[name];
+      var value = style[name];
+      if (value == null) value = "";
+      element.style[name] = value;
     }
 }
 
@@ -228,10 +230,15 @@ function setText(element, text) {
 
 // Get the value of an <input> element
 function getInputValue(inputElement) {
-    if (inputElement.type.toUpperCase() == 'CHECKBOX' ||
-        inputElement.type.toUpperCase() == 'RADIO')
-    {
-        return (inputElement.checked ? 'on' : 'off');
+    if (inputElement.type) {
+        if (inputElement.type.toUpperCase() == 'CHECKBOX' ||
+            inputElement.type.toUpperCase() == 'RADIO')
+        {
+            return (inputElement.checked ? 'on' : 'off');
+        }
+    }
+    if (inputElement.value == null) {
+        throw new SeleniumError("This element has no value; is it really a form field?");
     }
     return inputElement.value;
 }
@@ -277,7 +284,7 @@ function getKeyCodeFromKeySequence(keySequence) {
     if (match != null) {
         return match[0];
     }
-    throw SeleniumError("invalid keySequence");
+    throw new SeleniumError("invalid keySequence");
 }
 
 function createEventObject(element, controlKeyDown, altKeyDown, shiftKeyDown, metaKeyDown) {
@@ -312,78 +319,13 @@ function triggerKeyEvent(element, eventType, keySequence, canBubble, controlKeyD
 
             evt.initUIEvent(eventType, true, true, window, 1);
             evt.keyCode = keycode;
+            evt.which = keycode;
         }
 
         element.dispatchEvent(evt);
     }
 }
 
-/* Fire a mouse event in a browser-compatible manner */
-function triggerMouseEvent(element, eventType, canBubble, clientX, clientY, controlKeyDown, altKeyDown, shiftKeyDown, metaKeyDown) {
-    clientX = clientX ? clientX : 0;
-    clientY = clientY ? clientY : 0;
-
-    LOG.warn("triggerMouseEvent assumes setting screenX and screenY to 0 is ok");
-    var screenX = 0;
-    var screenY = 0;
-
-    canBubble = (typeof(canBubble) == undefined) ? true : canBubble;
-    if (element.fireEvent) {
-        LOG.info("element has fireEvent");
-        var evt = createEventObject(element, controlKeyDown, altKeyDown, shiftKeyDown, metaKeyDown);
-        evt.detail = 0;
-        evt.button = 1;
-        evt.relatedTarget = null;
-        if (!screenX && !screenY && !clientX && !clientY) {
-            element.fireEvent('on' + eventType);
-        }
-        else {
-            evt.screenX = screenX;
-            evt.screenY = screenY;
-            evt.clientX = clientX;
-            evt.clientY = clientY;
-
-            // when we go this route, window.event is never set to contain the event we have just created.
-            // ideally we could just slide it in as follows in the try-block below, but this normally
-            // doesn't work.  This is why I try to avoid this code path, which is only required if we need to
-            // set attributes on the event (e.g., clientX).
-            try {
-                window.event = evt;
-            }
-            catch(e) {
-                // getting an "Object does not support this action or property" error.  Save the event away
-                // for future reference.
-                // TODO: is there a way to update window.event?
-
-                // work around for http://jira.openqa.org/browse/SEL-280 -- make the event available somewhere:
-                selenium.browserbot.getCurrentWindow().selenium_event = evt;
-            }
-            element.fireEvent('on' + eventType, evt);
-        }
-    }
-    else {
-        LOG.info("element doesn't have fireEvent");
-        var evt = document.createEvent('MouseEvents');
-        if (evt.initMouseEvent)
-        {
-            LOG.info("element has initMouseEvent");
-            //Safari
-            evt.initMouseEvent(eventType, canBubble, true, document.defaultView, 1, screenX, screenY, clientX, clientY, controlKeyDown, altKeyDown, shiftKeyDown, metaKeyDown, 0, null)
-        }
-        else {
-        	LOG.warn("element doesn't have initMouseEvent; firing an event which should -- but doesn't -- have other mouse-event related attributes here, as well as controlKeyDown, altKeyDown, shiftKeyDown, metaKeyDown");
-            evt.initEvent(eventType, canBubble, true);
-            
-            evt.shiftKey = shiftKeyDown;
-            evt.metaKey = metaKeyDown;
-            evt.altKey = altKeyDown;
-            evt.ctrlKey = controlKeyDown;
-
-        }
-        element.dispatchEvent(evt);
-    }
-}
-
 function removeLoadListener(element, command) {
     LOG.info('Removing loadListenter for ' + element + ', ' + command);
     if (window.removeEventListener)
@@ -423,6 +365,145 @@ function getDocumentBase(doc) {
     return "";
 }
 
+function getTagName(element) {
+    var tagName;
+    if (element && element.tagName && element.tagName.toLowerCase) {
+        tagName = element.tagName.toLowerCase();
+    }
+    return tagName;
+}
+
+function absolutify(url, baseUrl) {
+    /** returns a relative url in its absolute form, given by baseUrl.
+    * 
+    * This function is a little odd, because it can take baseUrls that
+    * aren't necessarily directories.  It uses the same rules as the HTML 
+    * &lt;base&gt; tag; if the baseUrl doesn't end with "/", we'll assume
+    * that it points to a file, and strip the filename off to find its
+    * base directory.
+    *
+    * So absolutify("foo", "http://x/bar") will return "http://x/foo" (stripping off bar),
+    * whereas absolutify("foo", "http://x/bar/") will return "http://x/bar/foo" (preserving bar).
+    * Naturally absolutify("foo", "http://x") will return "http://x/foo", appropriately.
+    * 
+    * @param url the url to make absolute; if this url is already absolute, we'll just return that, unchanged
+    * @param baseUrl the baseUrl from which we'll absolutify, following the rules above.
+    * @return 'url' if it was already absolute, or the absolutized version of url if it was not absolute.
+    */
+    
+    // DGF isn't there some library we could use for this?
+        
+    if (/^\w+:/.test(url)) {
+        // it's already absolute
+        return url;
+    }
+    
+    var loc;
+    try {
+        loc = parseUrl(baseUrl);
+    } catch (e) {
+        // is it an absolute windows file path? let's play the hero in that case
+        if (/^\w:\\/.test(baseUrl)) {
+            baseUrl = "file:///" + baseUrl.replace(/\\/g, "/");
+            loc = parseUrl(baseUrl);
+        } else {
+            throw new SeleniumError("baseUrl wasn't absolute: " + baseUrl);
+        }
+    }
+    loc.search = null;
+    loc.hash = null;
+    
+    // if url begins with /, then that's the whole pathname
+    if (/^\//.test(url)) {
+        loc.pathname = url;
+        var result = reassembleLocation(loc);
+        return result;
+    }
+    
+    // if pathname is null, then we'll just append "/" + the url
+    if (!loc.pathname) {
+        loc.pathname = "/" + url;
+        var result = reassembleLocation(loc);
+        return result;
+    }
+    
+    // if pathname ends with /, just append url
+    if (/\/$/.test(loc.pathname)) {
+        loc.pathname += url;
+        var result = reassembleLocation(loc);
+        return result;
+    }
+    
+    // if we're here, then the baseUrl has a pathname, but it doesn't end with /
+    // in that case, we replace everything after the final / with the relative url
+    loc.pathname = loc.pathname.replace(/[^\/\\]+$/, url);
+    var result = reassembleLocation(loc);
+    return result;
+    
+}
+
+var URL_REGEX = /^((\w+):\/\/)(([^:]+):?([^@]+)?@)?([^\/\?:]*):?(\d+)?(\/?[^\?#]+)?\??([^#]+)?#?(.+)?/;
+
+function parseUrl(url) {
+    var fields = ['url', null, 'protocol', null, 'username', 'password', 'host', 'port', 'pathname', 'search', 'hash'];
+    var result = URL_REGEX.exec(url);
+    if (!result) {
+        throw new SeleniumError("Invalid URL: " + url);
+    }
+    var loc = new Object();
+    for (var i = 0; i < fields.length; i++) {
+        var field = fields[i];
+        if (field == null) {
+            continue;
+        }
+        loc[field] = result[i];
+    }
+    return loc;
+}
+
+function reassembleLocation(loc) {
+    if (!loc.protocol) {
+        throw new Error("Not a valid location object: " + o2s(loc));
+    }
+    var protocol = loc.protocol;
+    protocol = protocol.replace(/:$/, "");
+    var url = protocol + "://";
+    if (loc.username) {
+        url += loc.username;
+        if (loc.password) {
+            url += ":" + loc.password;
+        }
+        url += "@";
+    }
+    if (loc.host) {
+        url += loc.host;
+    }
+    
+    if (loc.port) {
+        url += ":" + loc.port;
+    }
+    
+    if (loc.pathname) {
+        url += loc.pathname;
+    }
+    
+    if (loc.search) {
+        url += "?" + loc.search;
+    }
+    if (loc.hash) {
+        var hash = loc.hash;
+        hash = loc.hash.replace(/^#/, "");
+        url += "#" + hash;
+    }
+    return url;
+}
+
+function canonicalize(url) {
+    var tempLink = window.document.createElement("link");
+    tempLink.href = url; // this will canonicalize the href
+    return tempLink.href;
+}
+
 function extractExceptionMessage(ex) {
     if (ex == null) return "null exception";
     if (ex.message != null) return ex.message;
@@ -630,24 +711,23 @@ function SeleniumError(message) {
     return error;
 }
 
-var Effect = new Object();
-
-objectExtend(Effect, {
-    highlight : function(element) {
-        var highLightColor = "yellow";
-        if (element.originalColor == undefined) { // avoid picking up highlight
-            element.originalColor = elementGetStyle(element, "background-color");
-        }
-        elementSetStyle(element, {"background-color" : highLightColor});
-        window.setTimeout(function() {
+function highlight(element) {
+    var highLightColor = "yellow";
+    if (element.originalColor == undefined) { // avoid picking up highlight
+        element.originalColor = elementGetStyle(element, "background-color");
+    }
+    elementSetStyle(element, {"backgroundColor" : highLightColor});
+    window.setTimeout(function() {
+        try {
             //if element is orphan, probably page of it has already gone, so ignore
             if (!element.parentNode) {
                 return;
             }
-            elementSetStyle(element, {"background-color" : element.originalColor});
-        }, 200);
-    }
-});
+            elementSetStyle(element, {"backgroundColor" : element.originalColor});
+        } catch (e) {} // DGF unhighlighting is very dangerous and low priority
+    }, 200);
+}
+
 
 
 // for use from vs.2003 debugger
@@ -663,7 +743,7 @@ function o2s(obj) {
 
 var seenReadyStateWarning = false;
 
-function openSeparateApplicationWindow(url) {
+function openSeparateApplicationWindow(url, suppressMozillaWarning) {
     // resize the Selenium window itself
     window.resizeTo(1200, 500);
     window.moveTo(window.screenX, 0);
@@ -688,7 +768,7 @@ function openSeparateApplicationWindow(url) {
     }
 
 
-    if (window.document.readyState == null && !seenReadyStateWarning) {
+    if (!suppressMozillaWarning && window.document.readyState == null && !seenReadyStateWarning) {
         alert("Beware!  Mozilla bug 300992 means that we can't always reliably detect when a new page has loaded.  Install the Selenium IDE extension or the readyState extension available from selenium.openqa.org to make page load detection more reliable.");
         seenReadyStateWarning = true;
     }
@@ -743,6 +823,11 @@ objectExtend(URLConfiguration.prototype, {
 
     isMultiWindowMode:function() {
         return this._isQueryParameterTrue('multiWindow');
+    },
+    
+    getBaseUrl:function() {
+        return this._getQueryParameter('baseUrl');
+            
     }
 });
 
